@@ -1,4 +1,4 @@
-# **************************************************************************
+# *****************************************************************************
 # *
 # * Authors:     Federico P. de Isidro Gomez (fp.deisidro@cnb.csic.es) [1]
 # *
@@ -6,7 +6,7 @@
 # *
 # * This program is free software; you can redistribute it and/or modify
 # * it under the terms of the GNU General Public License as published by
-# * the Free Software Foundation; either version 2 of the License, or
+# * the Free Software Foundation; either version 3 of the License, or
 # * (at your option) any later version.
 # *
 # * This program is distributed in the hope that it will be useful,
@@ -22,16 +22,18 @@
 # *  All comments concerning this program package may be sent to the
 # *  e-mail address 'scipion@cnb.csic.es'
 # *
-# **************************************************************************
+# *****************************************************************************
 
 import os
+
 from pyworkflow import BETA
 from pyworkflow.object import Set
 import pyworkflow.protocol.params as params
 import pyworkflow.utils.path as path
 import tomo.objects as tomoObj
-from imod import Plugin, utils
-from imod.protocols.protocol_base import ProtImodBase
+
+from .. import Plugin, utils
+from .protocol_base import ProtImodBase
 
 
 class ProtImodCtfCorrection(ProtImodBase):
@@ -52,59 +54,77 @@ class ProtImodCtfCorrection(ProtImodBase):
                       params.PointerParam,
                       label="Input tilt-series",
                       pointerClass='SetOfTiltSeries',
-                      help='Select the set of tilt-series to be CTF corrected.')
+                      help='Select the set of tilt-series to be '
+                           'CTF corrected. Usually this will be the '
+                           'tilt-series with alignment information.')
 
         form.addParam('inputSetOfCtfTomoSeries',
                       params.PointerParam,
-                      label="input tilt-series CTF estimation",
+                      label="Input CTF estimation",
                       pointerClass='SetOfCTFTomoSeries',
-                      help='Select the CTF estimation from the set of tilt-series.')
+                      help='Select the CTF estimation for the set '
+                           'of tilt-series.')
 
         form.addParam('defocusTol',
                       params.FloatParam,
                       label='Defocus tolerance (nm)',
                       default=200,
                       important=True,
-                      help='The value introduced must be the same used for CTF estimation in case ti has been '
-                           'performed with IMOD. \n\n'
-                           'Defocus tolerance in nanometers defining the center strips. The center strips are taken '
-                           'from the central region of a view that has defocus difference less than this tolerance. '
-                           'These kind of center strips from all views within AngleRange are considered to have a '
-                           'constant defocus and are used to compute the initial CTF after being further tessellated '
+                      help='The value introduced must be the same used for '
+                           'CTF estimation with IMOD.\n\n'
+                           'Defocus tolerance in nanometers defining the '
+                           'center strips. The center strips are taken '
+                           'from the central region of a view that has defocus '
+                           'difference less than this tolerance. '
+                           'These kind of center strips from all views within '
+                           'AngleRange are considered to have a '
+                           'constant defocus and are used to compute the '
+                           'initial CTF after being further tessellated '
                            'into tiles.')
 
         form.addParam('interpolationWidth',
                       params.IntParam,
-                      label='Interpolation Width (pixels)',
+                      label='Interpolation width (px)',
                       default='15',
                       important=True,
-                      help="The distance in pixels between the center lines of two consecutive strips. A pixel inside "
-                           "the region between those two center lines resides in both strips. As the two strips are "
-                           "corrected separately, that pixel will have 2 corrected values. The final value for that "
-                           "pixel is a linear interpolation of the 2 corrected values. If a value of 1 is entered, "
-                           "there is no such interpolation.  For a value greater than one, the entered value will be "
-                           "used whenever the strip width is less than 256 (i.e., at high tilt), and the value will be "
-                           "scaled proportional to the strip width for widths above 256.  This scaling keeps the "
-                           "computational time down and is reasonable because the defocus difference between adjacent "
-                           "wide strips at wider intervals is still less than that between the narrower strips at high "
-                           "tilt. However, strips at constant spacing can still be obtained by entering the negative "
-                           "of the desired spacing, which disables the scaling of the spacing.")
+                      help="The distance in pixels between the center lines "
+                           "of two consecutive strips. A pixel inside the "
+                           "region between those two center lines resides in "
+                           "both strips. As the two strips are corrected "
+                           "separately, that pixel will have 2 corrected "
+                           "values. The final value for that pixel is a "
+                           "linear interpolation of the 2 corrected "
+                           "values. If a value of 1 is entered, there is "
+                           "no such interpolation. For a value greater "
+                           "than one, the entered value will be used "
+                           "whenever the strip width is less than 256 "
+                           "(i.e., at high tilt), and the value will be "
+                           "scaled proportional to the strip width for widths "
+                           "above 256.  This scaling keeps the computational "
+                           "time down and is reasonable because the defocus "
+                           "difference between adjacent wide strips at "
+                           "wider intervals is still less than that between "
+                           "the narrower strips at high tilt. However, strips "
+                           "at constant spacing can still be obtained by "
+                           "entering the negative of the desired spacing, "
+                           "which disables the scaling of the spacing.")
 
         form.addHidden(params.USE_GPU,
                        params.BooleanParam,
                        default=True,
                        label="Use GPU for execution",
-                       help="This protocol has both CPU and GPU implementation.\
-                       Select the one you want to use.")
+                       help="This protocol has both CPU and GPU implementation."
+                            "Select the one you want to use.")
 
         form.addHidden(params.GPU_LIST,
                        params.StringParam,
                        default='0',
-                       expertLevel=params.LEVEL_ADVANCED,
                        label="Choose GPU IDs",
-                       help="GPU ID. To pick the best available one set 0. For a specific GPU set its number ID.")
+                       help="GPU ID. To pick the best available one set 0. "
+                            "For a specific GPU set its number ID "
+                            "(starting from 1).")
 
-    # -------------------------- INSERT steps functions ---------------------
+    # -------------------------- INSERT steps functions -----------------------
     def _insertAllSteps(self):
         for ts in self.inputSetOfTiltSeries.get():
             self._insertFunctionStep(self.convertInputStep, ts.getObjId())
@@ -113,12 +133,16 @@ class ProtImodCtfCorrection(ProtImodBase):
             self._insertFunctionStep(self.createOutputStep, ts.getObjId())
         self._insertFunctionStep(self.closeOutputSetsStep)
 
-    # --------------------------- STEPS functions ----------------------------
+    # --------------------------- STEPS functions -----------------------------
+    def convertInputStep(self, tsObjId):
+        # Considering swapXY is required to make tilt axis vertical
+        super().convertInputStep(tsObjId, doSwap=True)
+
     def generateDefocusFile(self, tsObjId):
         ts = self.inputSetOfTiltSeries.get()[tsObjId]
         tsId = ts.getTsId()
 
-        self.debug("Generating defocus file for %s (OBjId), %s (TsId)"% (tsObjId, tsId))
+        self.debug(f"Generating defocus file for {tsObjId} (ObjId), {tsId} (TsId)")
 
         # Compose the defocus file path
         defocusFilePath = self.getDefocusFileName(ts)
@@ -128,7 +152,8 @@ class ProtImodCtfCorrection(ProtImodBase):
         utils.generateDefocusIMODFileFromObject(ctfTomoSeries, defocusFilePath)
 
     def getDefocusFileName(self, ts):
-        """ Returns the path of the defocus filename based on the tilt series and creates the folder/s"""
+        """ Returns the path of the defocus filename based on
+         the tilt series and creates the folder/s"""
 
         tmpPrefix = self._getTmpPath(ts.getTsId())
         path.makePath(tmpPrefix)
@@ -151,9 +176,9 @@ class ProtImodCtfCorrection(ProtImodBase):
             'voltage': self.inputSetOfTiltSeries.get().getAcquisition().getVoltage(),
             'sphericalAberration': self.inputSetOfTiltSeries.get().getAcquisition().getSphericalAberration(),
             'defocusTol': self.defocusTol.get(),
-            'pixelSize': self.inputSetOfTiltSeries.get().getSamplingRate()/10,
+            'pixelSize': self.inputSetOfTiltSeries.get().getSamplingRate() / 10,
             'amplitudeContrast': self.inputSetOfTiltSeries.get().getAcquisition().getAmplitudeContrast(),
-            'interpolationWidth': self.interpolationWidth.get(),
+            'interpolationWidth': self.interpolationWidth.get()
         }
 
         argsCtfPhaseFlip = "-InputStack %(inputStack)s " \
@@ -168,16 +193,17 @@ class ProtImodCtfCorrection(ProtImodBase):
                            "-InterpolationWidth %(interpolationWidth)d "
 
         if self.usesGpu():
-            paramsCtfPhaseFlip.update({
-                "useGPU": self.getGpuList()[0]
-            })
+            argsCtfPhaseFlip += f"-UseGPU {self.getGpuList()[0]} " \
+                                "-ActionIfGPUFails 2,2 "
 
-            argsCtfPhaseFlip += "-UseGPU %(useGPU)d "
+        if ts.getFirstItem().hasTransform():
+            paramsCtfPhaseFlip['xformFile'] = os.path.join(tmpPrefix, ts.getFirstItem().parseFileName(extension=".xf"))
+            argsCtfPhaseFlip += "-TransformFile %(xformFile)s "
 
         Plugin.runImod(self, 'ctfphaseflip', argsCtfPhaseFlip % paramsCtfPhaseFlip)
 
     def createOutputStep(self, tsObjId):
-        output =self.getOutputSetOfTiltSeries(self.inputSetOfTiltSeries.get())
+        output = self.getOutputSetOfTiltSeries(self.inputSetOfTiltSeries.get())
 
         ts = self.inputSetOfTiltSeries.get()[tsObjId]
         tsId = ts.getTsId()
@@ -185,6 +211,7 @@ class ProtImodCtfCorrection(ProtImodBase):
 
         newTs = tomoObj.TiltSeries(tsId=tsId)
         newTs.copyInfo(ts)
+        newTs.setCtfCorrected(True)
         output.append(newTs)
 
         for index, tiltImage in enumerate(ts):
@@ -192,7 +219,8 @@ class ProtImodCtfCorrection(ProtImodBase):
             newTi.copyInfo(tiltImage, copyId=True, copyTM=False)
             newTi.setAcquisition(tiltImage.getAcquisition())
             newTi.setLocation(index + 1,
-                              (os.path.join(extraPrefix, tiltImage.parseFileName())))
+                              (os.path.join(extraPrefix,
+                                            tiltImage.parseFileName())))
             newTs.append(newTi)
 
         newTs.write(properties=False)
@@ -205,18 +233,31 @@ class ProtImodCtfCorrection(ProtImodBase):
         self.TiltSeries.write()
         self._store()
 
-    # --------------------------- UTILS functions ----------------------------
+    # --------------------------- UTILS functions -----------------------------
     def getCtfTomoSeriesFromTsId(self, setOfCtfTomoSeries, tsId):
         for ctfTomoSeries in self.inputSetOfCtfTomoSeries.get():
             if tsId == ctfTomoSeries.getTsId():
                 return ctfTomoSeries
 
-    # --------------------------- INFO functions ----------------------------
+    # --------------------------- INFO functions ------------------------------
+    def _warnings(self):
+        warnings = []
+        ts = self.inputSetOfTiltSeries.get()
+        if not ts.getFirstItem().getFirstItem().hasTransform():
+            warnings.append("Input tilt-series do not have alignment "
+                            "information! The recommended workflow is to "
+                            "estimate CTF on raw tilt-series and then here "
+                            "provide tilt-series with alignment "
+                            "(non-interpolated).")
+
+        return warnings
+
     def _validate(self):
         validateMsgs = []
 
         if self.inputSetOfTiltSeries.get().getSize() != self.inputSetOfCtfTomoSeries.get().getSize():
-            validateMsgs.append("Input set of tilt-series and input set of CTF tomo estimations must contain the "
+            validateMsgs.append("Input tilt-series and CTF tomo "
+                                "estimations must contain the "
                                 "same number of elements.")
 
         return validateMsgs
@@ -224,19 +265,17 @@ class ProtImodCtfCorrection(ProtImodBase):
     def _summary(self):
         summary = []
         if self.TiltSeries:
-            summary.append("Input Tilt-Series: %d.\nCTF corrections applied: %d.\n"
+            summary.append("Input tilt-series: %d\nCTF corrections applied: %d"
                            % (self.inputSetOfCtfTomoSeries.get().getSize(),
                               self.TiltSeries.getSize()))
         else:
-            summary.append("Output classes not ready yet.")
+            summary.append("Outputs are not ready yet.")
         return summary
 
     def _methods(self):
         methods = []
         if self.TiltSeries:
-            methods.append("%d Tilt-series have been CTF corrected using the IMOD ctfphaseflip software.\n"
+            methods.append("%d tilt-series have been CTF corrected "
+                           "using the IMOD *ctfphaseflip* program."
                            % (self.TiltSeries.getSize()))
-        else:
-            methods.append("Output classes not ready yet.")
         return methods
-
