@@ -1,4 +1,4 @@
-# **************************************************************************
+# *****************************************************************************
 # *
 # * Authors:     Federico P. de Isidro Gomez (fp.deisidro@cnb.csic.es) [1]
 # *
@@ -6,7 +6,7 @@
 # *
 # * This program is free software; you can redistribute it and/or modify
 # * it under the terms of the GNU General Public License as published by
-# * the Free Software Foundation; either version 2 of the License, or
+# * the Free Software Foundation; either version 3 of the License, or
 # * (at your option) any later version.
 # *
 # * This program is distributed in the hope that it will be useful,
@@ -22,19 +22,19 @@
 # *  All comments concerning this program package may be sent to the
 # *  e-mail address 'scipion@cnb.csic.es'
 # *
-# **************************************************************************
+# *****************************************************************************
 
 import os
+
 from pyworkflow import BETA
 import pyworkflow.protocol.params as params
-from pyworkflow.utils import path
+from pyworkflow.utils import path, removeBaseExt
 from pyworkflow.object import Set
-from pyworkflow.protocol.constants import STEPS_PARALLEL
 import tomo.objects as tomoObj
 import tomo.constants as constants
-from imod import Plugin
-from imod import utils
-from imod.protocols.protocol_base import ProtImodBase
+
+from .. import Plugin, utils
+from .protocol_base import ProtImodBase
 
 
 class ProtImodGoldBeadPicker3d(ProtImodBase):
@@ -54,13 +54,16 @@ class ProtImodGoldBeadPicker3d(ProtImodBase):
                       params.PointerParam,
                       pointerClass='SetOfTomograms',
                       important=True,
-                      label='Input set of tilt-series',
-                      help='Input set of tomograms from which gold beads will be picked.')
+                      label='Input set of tomograms',
+                      help='Input set of tomograms from which gold beads '
+                           'will be picked. A tomogram needs to be thicker '
+                           'than normal because the program cannot find '
+                           'beads too close to the surfaces of a tomogram.')
 
         form.addParam('beadDiameter',
                       params.FloatParam,
-                      label='Fiducial diameter (pixels)',
-                      default='10',
+                      label='Fiducials diameter (px)',
+                      default='18',
                       help="Diameter of beads in pixels.")
 
         form.addParam('beadsColor',
@@ -75,26 +78,34 @@ class ProtImodGoldBeadPicker3d(ProtImodBase):
 
         form.addParam('minRelativeStrength',
                       params.FloatParam,
-                      label='Minimum relative Strength',
+                      expertLevel=params.LEVEL_ADVANCED,
+                      label='Minimum relative strength',
                       default=0.05,
-                      help='Minimum relative peak strength for keeping a peak in the analysis.  The square root of the '
-                           'specified value is used for comparing with the square root of peak strength, for '
-                           'compatibility with existing command files. The default is 0.05, which corresponds to a '
-                           'relative square root peak strength of 0.22.  Too many weak peaks can prevent a dip from '
-                           'showing up in the smoothed histogram of strengths.  If the program fails to find a '
-                           'histogram dip, one strategy is to try raising this value.')
+                      help='Minimum relative peak strength for keeping a '
+                           'peak in the analysis.  The square root of the '
+                           'specified value is used for comparing with the '
+                           'square root of peak strength, for compatibility '
+                           'with existing command files. The default is 0.05, '
+                           'which corresponds to a relative square root peak '
+                           'strength of 0.22. Too many weak peaks can prevent '
+                           'a dip from showing up in the smoothed histogram '
+                           'of strengths.  If the program fails to find a '
+                           'histogram dip, one strategy is to try raising '
+                           'this value.')
 
         form.addParam('minSpacing',
                       params.FloatParam,
+                      expertLevel=params.LEVEL_ADVANCED,
                       label='Minimum spacing',
                       default=0.9,
-                      help='Minimum spacing between peaks as a fraction of the bead size. When two peaks are closer '
-                           'than this distance apart, the weaker one is eliminated unless the -both option is entered. '
-                           'The default is 0.9.  A value less than 1 is helpful for picking both beads in a pair.')
+                      help='Minimum spacing between peaks as a fraction of '
+                           'the bead size. When two peaks are closer '
+                           'than this distance apart, the weaker one is '
+                           'eliminated unless the -both option is entered. '
+                           'The default is 0.9. A value less than 1 is '
+                           'helpful for picking both beads in a pair.')
 
-        form.addParallelSection(threads=4, mpi=1)
-
-    # -------------------------- INSERT steps functions ---------------------
+    # -------------------------- INSERT steps functions -----------------------
     def _insertAllSteps(self):
         self.defineExecutionPararell()
 
@@ -118,19 +129,17 @@ class ProtImodGoldBeadPicker3d(ProtImodBase):
         self._insertFunctionStep('closeOutputSetStep',
                                  prerequisites=allOutputId)
 
-    # --------------------------- STEPS functions ----------------------------
+    # --------------------------- STEPS functions -----------------------------
     def pickGoldBeadsStep(self, tsObjId):
         tomo = self.inputSetOfTomograms.get()[tsObjId]
-        location = tomo.getLocation()[1]
-        fileName, _ = os.path.splitext(location)
-
-        extraPrefix = self._getExtraPath(os.path.basename(fileName))
+        fileName = removeBaseExt(tomo.getFileName())
+        extraPrefix = self._getExtraPath(fileName)
         path.makePath(extraPrefix)
 
         """ Run findbeads3d IMOD program """
         paramsFindbeads3d = {
-            'inputFile': location,
-            'outputFile': os.path.join(extraPrefix, "%s.mod" % os.path.basename(fileName)),
+            'inputFile': tomo.getFileName(),
+            'outputFile': os.path.join(extraPrefix, "%s.mod" % fileName),
             'beadSize': self.beadDiameter.get(),
             'minRelativeStrength': self.minRelativeStrength.get(),
             'minSpacing': self.minSpacing.get(),
@@ -140,7 +149,8 @@ class ProtImodGoldBeadPicker3d(ProtImodBase):
                           "-OutputFile %(outputFile)s " \
                           "-BeadSize %(beadSize)d " \
                           "-MinRelativeStrength %(minRelativeStrength)f " \
-                          "-MinSpacing %(minSpacing)d "
+                          "-StorageThreshold 0.0 " \
+                          "-MinSpacing %(minSpacing)f "
 
         if self.beadsColor.get() == 1:
             argsFindbeads3d += "-LightBeads "
@@ -149,7 +159,7 @@ class ProtImodGoldBeadPicker3d(ProtImodBase):
 
     def convertModelToCoordinatesStep(self, tsObjId):
         tomo = self.inputSetOfTomograms.get()[tsObjId]
-        location = tomo.getLocation()[1]
+        location = tomo.getFileName()
         fileName, _ = os.path.splitext(location)
 
         extraPrefix = self._getExtraPath(os.path.basename(fileName))
@@ -161,21 +171,23 @@ class ProtImodGoldBeadPicker3d(ProtImodBase):
         }
 
         argsModel2Point = "-InputFile %(inputFile)s " \
-                          "-OutputFile %(outputFile)s " \
+                          "-OutputFile %(outputFile)s "
 
         Plugin.runImod(self, 'model2point', argsModel2Point % paramsModel2Point)
 
     def createOutputStep(self, tsObjId):
         tomo = self.inputSetOfTomograms.get()[tsObjId]
-        location = tomo.getLocation()[1]
+        location = tomo.getFileName()
         fileName, _ = os.path.splitext(location)
 
         extraPrefix = self._getExtraPath(os.path.basename(fileName))
 
         """ Create the output set of coordinates 3D from gold beads detected """
-        output = self.getOutputSetOfCoordinates3Ds(self.inputSetOfTomograms.get(), self.inputSetOfTomograms.get())
+        output = self.getOutputSetOfCoordinates3Ds(self.inputSetOfTomograms.get(),
+                                                   self.inputSetOfTomograms.get())
 
-        coordFilePath = os.path.join(extraPrefix, "%s.xyz" % os.path.basename(fileName))
+        coordFilePath = os.path.join(extraPrefix,
+                                     "%s.xyz" % os.path.basename(fileName))
 
         coordList = utils.formatGoldBead3DCoordinatesList(coordFilePath)
 
@@ -191,6 +203,7 @@ class ProtImodGoldBeadPicker3d(ProtImodBase):
                 output.append(newCoord3D)
                 output.update(newCoord3D)
 
+            output.setBoxSize(self.beadDiameter.get())
             output.write()
 
         self._store()
