@@ -1,4 +1,4 @@
-# **************************************************************************
+# *****************************************************************************
 # *
 # * Authors:     Federico P. de Isidro Gomez (fp.deisidro@cnb.csic.es) [1]
 # *
@@ -6,7 +6,7 @@
 # *
 # * This program is free software; you can redistribute it and/or modify
 # * it under the terms of the GNU General Public License as published by
-# * the Free Software Foundation; either version 2 of the License, or
+# * the Free Software Foundation; either version 3 of the License, or
 # * (at your option) any later version.
 # *
 # * This program is distributed in the hope that it will be useful,
@@ -22,19 +22,17 @@
 # *  All comments concerning this program package may be sent to the
 # *  e-mail address 'scipion@cnb.csic.es'
 # *
-# **************************************************************************
+# *****************************************************************************
 
 import os
-from pwem.protocols import EMProtocol
+
 from pyworkflow import BETA
-import pyworkflow.utils.path as path
 import pyworkflow.protocol.params as params
 from pyworkflow.object import Set
-from tomo.protocols import ProtTomoBase
 import tomo.objects as tomoObj
-import imod.utils as utils
-from imod import Plugin
-from imod.protocols.protocol_base import ProtImodBase, OUTPUT_TILTSERIES_NAME
+
+from .. import Plugin
+from .protocol_base import ProtImodBase, OUTPUT_TILTSERIES_NAME
 
 
 class ProtImodXraysEraser(ProtImodBase):
@@ -60,47 +58,56 @@ class ProtImodXraysEraser(ProtImodBase):
                       params.PointerParam,
                       pointerClass='SetOfTiltSeries',
                       important=True,
-                      label='Input set of tilt-series.')
+                      label='Input set of tilt-series')
 
         form.addParam('peakCriterion',
                       params.FloatParam,
                       default=8.0,
                       label='Peak criterion',
                       expertLevel=params.LEVEL_ADVANCED,
-                      help='Criterion # of SDs above local mean for erasing peak based on intensity (the default is 10 '
-                           'SDs)')
+                      help='Criterion # of SDs above local mean for erasing '
+                           'peak based on intensity (the default is 8 SDs)')
 
         form.addParam('diffCriterion',
                       params.FloatParam,
                       default=6.0,
                       label='Difference criterion',
                       expertLevel=params.LEVEL_ADVANCED,
-                      help='Criterion # of SDs above mean pixel-to-pixel difference for erasing a peak based on '
-                           'differences (the default is 10 SDs).')
+                      help='Criterion # of SDs above mean pixel-to-pixel '
+                           'difference for erasing a peak based on '
+                           'differences (the default is 6 SDs).')
 
         form.addParam('maximumRadius',
                       params.FloatParam,
                       default=4.2,
-                      label='Maximum radius (pixels)',
+                      label='Maximum radius (px)',
                       expertLevel=params.LEVEL_ADVANCED,
-                      help='Maximum radius of peak area to erase (the default is 2.1 pixels).')
+                      help='Maximum radius of peak area to erase (the '
+                           'default is 4.2 pixels).')
 
         form.addParam('bigDiffCriterion',
                       params.IntParam,
                       default=19,
                       label='Big difference criterion',
                       expertLevel=params.LEVEL_ADVANCED,
-                      help='An extra-large peak will be erased only if the value for the maximum difference between '
-                           'adjacent pixels, averaged over the most extreme one-fourth of the pixels in the patch, '
-                           'exceeds this criterion, evaluated as the number of SDs above the mean absolute difference '
-                           'between adjacent pixels in the scan area.  The default is 19.  This high a value is needed '
-                           'to prevent gold erasure on low-noise data sets with small gold particles, and a lower '
-                           'value may be needed to make extra-large peak removal useful.')
+                      help='An extra-large peak will be erased only if the '
+                           'value for the maximum difference between '
+                           'adjacent pixels, averaged over the most extreme '
+                           'one-fourth of the pixels in the patch, '
+                           'exceeds this criterion, evaluated as the '
+                           'number of SDs above the mean absolute difference '
+                           'between adjacent pixels in the scan area. The '
+                           'default is 19.  This high a value is needed '
+                           'to prevent gold erasure on low-noise data sets '
+                           'with small gold particles, and a lower value '
+                           'may be needed to make extra-large peak removal '
+                           'useful.')
 
-    # -------------------------- INSERT steps functions ---------------------
+    # -------------------------- INSERT steps functions -----------------------
     def _insertAllSteps(self):
         for ts in self.inputSetOfTiltSeries.get():
-            self._insertFunctionStep(self.convertInputStep, ts.getObjId(), generateAngleFile=False)
+            self._insertFunctionStep(self.convertInputStep, ts.getObjId(),
+                                     generateAngleFile=False)
             self._insertFunctionStep(self.eraseXraysStep, ts.getObjId())
             self._insertFunctionStep(self.createOutputStep, ts.getObjId())
         self._insertFunctionStep(self.closeOutputStep)
@@ -118,16 +125,18 @@ class ProtImodXraysEraser(ProtImodBase):
             'findPeaks': 1,
             'peakCriterion': self.peakCriterion.get(),
             'diffCriterion': self.diffCriterion.get(),
-            'growCriterion': 4,
-            'scanCriterion': 3,
+            'growCriterion': 4.,
+            'scanCriterion': 3.,
             'maximumRadius': self.maximumRadius.get(),
-            'giantCriterion': 12,
-            'extraLargeRadius': 8,
+            'giantCriterion': 12.,
+            'extraLargeRadius': 8.,
             'bigDiffCriterion': self.bigDiffCriterion.get(),
             'annulusWidth': 2.0,
             'xyScanSize': 100,
             'edgeExclusionWidth': 4,
-            'pointModel': os.path.join(extraPrefix, ts.getFirstItem().parseFileName(suffix="_fid", extension=".mod")),
+            'pointModel': os.path.join(extraPrefix,
+                                       ts.getFirstItem().parseFileName(suffix="_fid",
+                                                                       extension=".mod")),
             'borderSize': 2,
             'polynomialOrder': 2,
         }
@@ -137,12 +146,12 @@ class ProtImodXraysEraser(ProtImodBase):
                         "-FindPeaks %(findPeaks)d " \
                         "-PeakCriterion %(peakCriterion).2f " \
                         "-DiffCriterion %(diffCriterion).2f " \
-                        "-GrowCriterion %(growCriterion)d " \
-                        "-ScanCriterion %(scanCriterion)d " \
+                        "-GrowCriterion %(growCriterion).2f " \
+                        "-ScanCriterion %(scanCriterion).2f " \
                         "-MaximumRadius %(maximumRadius).2f " \
-                        "-GiantCriterion %(giantCriterion)d " \
-                        "-ExtraLargeRadius %(extraLargeRadius)d " \
-                        "-BigDiffCriterion %(bigDiffCriterion)d " \
+                        "-GiantCriterion %(giantCriterion).2f " \
+                        "-ExtraLargeRadius %(extraLargeRadius).2f " \
+                        "-BigDiffCriterion %(bigDiffCriterion).2f " \
                         "-AnnulusWidth %(annulusWidth).2f " \
                         "-XYScanSize %(xyScanSize)d " \
                         "-EdgeExclusionWidth %(edgeExclusionWidth)d " \
@@ -167,7 +176,8 @@ class ProtImodXraysEraser(ProtImodBase):
             newTi.copyInfo(tiltImage, copyId=True, copyTM=True)
             newTi.setAcquisition(tiltImage.getAcquisition())
             newTi.setLocation(index + 1,
-                              (os.path.join(extraPrefix, tiltImage.parseFileName())))
+                              (os.path.join(extraPrefix,
+                                            tiltImage.parseFileName())))
             newTs.append(newTi)
 
         newTs.write(properties=False)
@@ -176,20 +186,19 @@ class ProtImodXraysEraser(ProtImodBase):
         self._store()
 
     def closeOutputStep(self):
-
-
         getattr(self, OUTPUT_TILTSERIES_NAME).setStreamState(Set.STREAM_CLOSED)
         self._store()
 
-    # --------------------------- INFO functions ----------------------------
+    # --------------------------- INFO functions ------------------------------
     def _summary(self):
         summary = []
         if self.TiltSeries:
-            summary.append("Input Tilt-Series: %d.\nX-rays erased output tilt series: %d.\n"
+            summary.append("Input tilt-series: %d\nX-rays erased output "
+                           "tilt series: %d"
                            % (self.inputSetOfTiltSeries.get().getSize(),
                               self.TiltSeries.getSize()))
         else:
-            summary.append("Output classes not ready yet.")
+            summary.append("Outputs are not ready yet.")
 
         return summary
 
@@ -197,8 +206,7 @@ class ProtImodXraysEraser(ProtImodBase):
         methods = []
         if self.TiltSeries:
             methods.append("The x-rays artifacts have been erased for %d "
-                           "Tilt-series using the IMOD program ccderaser.\n"
+                           "tilt-series using the IMOD *ccderaser* command.\n"
                            % (self.TiltSeries.getSize()))
 
         return methods
-
